﻿Type=Service
Version=2.52
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Private rv As RemoteViews
	
	Private mAppWidgetIdList As List
End Sub
Sub Service_Create
	rv = ConfigureHomeWidget("wid1", "Widget", 0, "WorldClock", True)
	
	mAppWidgetIdList.Initialize
End Sub

Sub Service_Start (StartingIntent As Intent)
	'The mAppWidgetIdList keeps track of what widgets we have to update.
	'If there is only one appWidgetId in the starting intent, the list contains just this id.
	'If there is a list of appWidgetIds in the starting intent, then we will fill the mAppWidgetIdList with them.
	'If there is no appWidgetId and no appWidgetId list in the starting intent then the
	'Sub Util.GetWidgetIds() tries to get all ids of the widgets created by this service.
	
	'Clear the list of AppWidgetIds
	mAppWidgetIdList.Clear
	
	'Is the service called from a single widget?
	If StartingIntent.HasExtra("appWidgetId") Then
		mAppWidgetIdList.Add(StartingIntent.GetExtra("appWidgetId"))
	End If
	
	Dim helper() As Int
	'Is there a list of widget ids in the Intent?
	If mAppWidgetIdList.Size = 0 Then
		If StartingIntent.HasExtra("appWidgetIds") Then
			helper = StartingIntent.GetExtra("appWidgetIds")
			mAppWidgetIdList.AddAll(helper)
		End If
	End If

	'Is the service called manually without any widget ids in starting intent?
	If mAppWidgetIdList.Size = 0 Then
		helper = Util.GetWidgetIds("advancedwidget")
		mAppWidgetIdList.AddAll(helper)
	End If
	
	'If there is no widget on the homescreen we can just exit
	If mAppWidgetIdList.Size = 0 Then
		Log("No AppWidget Ids found!")
		Return
	End If

	'Handle the widget events.
	If Not(rv.HandleWidgetEvents(StartingIntent)) Then
		'If the service is called from the configuration activity, the starting intent has a special action
		If StartingIntent.Action = "android.appwidget.action.APPWIDGET_UPDATE_OPTIONS" Then
			Log("Settings of Widget changed")
			Widget_UpdateSettings
		End If
		'If a widget instance is deleted we can delete its configuration
		If StartingIntent.Action = "android.appwidget.action.APPWIDGET_DELETED" Then
			Log("A widget instance is deleted")
			Widget_Deleted
		End If

		'If the time or timezone changed on the device
		If StartingIntent.Action = "android.intent.action.TIMEZONE_CHANGED" OR StartingIntent.Action = "android.intent.action.TIME_SET" Then
			Log("Time(Zone) has changed")
			Widget_RequestUpdate
		End If		
	End If

	'Reschedule the service every minute, so the clock gets updated correctly
	If StartingIntent.Action <> "android.appwidget.action.APPWIDGET_DISABLED" Then
		Dim si As Intent
		si.Initialize("android.appwidget.action.APPWIDGET_UPDATE", "")
		si.SetComponent(Util.GetPackageName & "/.advancedwidget")
		StartServiceAt(si, TimeToNextMinute, False)
	End If
End Sub

Sub Service_Destroy
	CancelScheduledService("")
End Sub

Sub Widget_RequestUpdate
	Dim LocationText As String
	Dim tzLocal As AHTimeZone
	tzLocal.Initialize

	'Instead of just updating the widget like in normal B4A code we need to loop over all widget ids and
	'set the content of each widget individually.
	'Here the setting information for each widget is read and the content is updated for each widget instance
	'individually.
	For i = 0 To mAppWidgetIdList.Size - 1
		Log("Update Widget: " & mAppWidgetIdList.Get(i))
		
		Dim layout As String

		Select StateManager.GetSetting2("layout_" & mAppWidgetIdList.Get(i), 0)
			Case 0
				layout = "advancedwidget_layout0"
			Case 1
				layout = "advancedwidget_layout1"
			Case 2
				layout = "advancedwidget_layout2"
			Case 3
				layout = "advancedwidget_layout3"
		End Select
		
		'Load the configured layout for the widget
		Util.SetWidgetLayout("advancedwidget", rv, layout)

		Dim vTextColor As Int
		vTextColor = StateManager.GetSetting2("textcolor_" & mAppWidgetIdList.Get(i), Colors.White)
		rv.SetTextColor("lLocation", vTextColor)
		rv.SetTextColor("lTimeZone", vTextColor)
		rv.SetTextColor("lClock", vTextColor)
		rv.SetTextColor("lDate", vTextColor)

		'Set special OnClick Listener for the pMain panel that supports AppWidgetId.
		'The standard B4A event does not add the appWidgetId to the starting intent so it is impossible to decide from
		'which widget the event was fired. The Util.SetWidgetClickEvent() sub fixes this.
		'If you have more than one view for which you will catch the click event just add them here.
		'Since the SetWidgetLayout generates a new RemoteViews object we have to always set the event again here.
		Util.SetWidgetClickEvent("advancedwidget", mAppWidgetIdList.Get(i), rv, "pMain", "pMain")
		
		'Set Widget Transparency
		Dim alpha As Int
		alpha = StateManager.GetSetting2("transp_" & mAppWidgetIdList.Get(i), 255)
		Util.SetRemoteViewAlpha("advancedwidget", "backimage", rv, alpha)
		
		Dim tz As AHTimeZone
		tz.Initialize2(StateManager.GetSetting2("timezone_" & mAppWidgetIdList.Get(i), tzLocal.ID))
		LocationText = StateManager.GetSetting2("cityname_" & mAppWidgetIdList.Get(i), "Default")
		
		rv.SetText("lLocation", LocationText)
		rv.SetText("lTimeZone", tz.ID)

		'Display the time in the widget
		Dim du As AHDateUtils
		rv.SetText("lClock", du.FormatDateTime(DateTime.Now+tz.GetOffset(DateTime.Now) , du.FORMAT_SHOW_TIME+du.FORMAT_UTC))
		rv.SetText("lDate", du.FormatDateTime(DateTime.Now+tz.GetOffset(DateTime.Now), du.FORMAT_SHOW_WEEKDAY+du.FORMAT_SHOW_DATE+du.FORMAT_ABBREV_WEEKDAY+du.FORMAT_UTC))
		'Just update one widget instance with this special sub
		Util.UpdateSingleWidget(rv, mAppWidgetIdList.Get(i))
	Next
End Sub

'The Settings of the widget have changed. Reconfigure your Widget here.
'Here we just do a normal update of the widget instance
Sub Widget_UpdateSettings
	Widget_RequestUpdate
End Sub

'Gets called when a single widget instance is deleted. You can delete the widget settings here.
Sub Widget_Deleted
	Log("Widget deleted: " & mAppWidgetIdList.Get(0))
	
	If mAppWidgetIdList.Size = 1 Then
		'Removing a setting is not supported by StateManager so we just set default values here
		Dim tzLocal As AHTimeZone
		tzLocal.Initialize
		StateManager.SetSetting("timezone_" & mAppWidgetIdList.Get(0), tzLocal.ID)
		StateManager.SetSetting("cityname_" & mAppWidgetIdList.Get(0), "Default")
		StateManager.SetSetting("layout_" & mAppWidgetIdList.Get(0), 0)
		StateManager.SetSetting("transp_" & mAppWidgetIdList.Get(0), 255)
	End If
End Sub

Sub Widget_Disabled
	StopService("")	
End Sub

'Event Sub for the main panel
Sub pMain_Clicked
	Log("Main Panel clicked => show config activity")
	'If we receive a click event, normally the mAppWidgetIdList should only contain one id.
	If mAppWidgetIdList.Size = 1 Then
		'We start the configuration activity again when the widget is clicked.
		'For this we need an intent which contains the correct action, component name and the
		'WidgetId and a special flag for detecting manual call of the activity in the intent extras
		Dim ConfigIntent As Intent
		ConfigIntent.Initialize("android.appwidget.action.APPWIDGET_CONFIGURE", "")
		ConfigIntent.SetComponent(Util.GetPackageName & "/.widgetconfig")
		ConfigIntent.PutExtra("appWidgetId", mAppWidgetIdList.Get(0))
		ConfigIntent.PutExtra("startedManually", 1)
		StartActivity(ConfigIntent)
	End If
End Sub

'Calculate the time to the next full minute
Sub TimeToNextMinute As Long
	Dim ret As Long

	ret = DateTime.Now + DateTime.TicksPerMinute
	ret = ret - (ret Mod DateTime.TicksPerMinute)	
	Return ret
End Sub