﻿Type=StaticCode
Version=2.52
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

End Sub

'Updates only a single widget instance.
'
' pRv - A RemoteViews object
' WidgetId - Id of the widget to update
Public Sub UpdateSingleWidget(pRv As RemoteViews, WidgetId As Int)
	Dim Obj1, Obj2 As Reflector
	Dim current As Object
	
	Obj1.Target = pRv ' a RemoteViewsWrapper
	
	Obj1.RunMethod("checkNull") 'does some internal checking and may set current
	current = Obj1.GetField("current") ' a RemoteViews - get this after checkNull
	Obj2.Target = Obj1.RunStaticMethod("android.appwidget.AppWidgetManager", "getInstance", Array As Object(Obj1.GetContext), Array As String("android.content.Context"))
	Obj2.RunMethod4("updateAppWidget", Array As Object(WidgetId, current), Array As String("java.lang.int", "android.widget.RemoteViews"))
	Obj1.SetField2("current", Null)
End Sub

'Get the Ids of all instances of this widget on the homescreen created by the given service
'
' ServiceName - The name of the WidgetService
Public Sub GetWidgetIds(ServiceName As String) As Int()
	Dim Obj1, Obj2 As Reflector
	Dim cn As Object
	
	Obj2.Target = Obj1.RunStaticMethod("android.appwidget.AppWidgetManager", "getInstance", Array As Object(Obj1.GetContext), Array As String("android.content.Context"))
	cn = Obj1.CreateObject2("android.content.ComponentName", Array As Object(Obj1.GetContext, Obj1.GetStaticField("anywheresoftware.b4a.BA", "packageName") & "." & ServiceName & "$" & ServiceName & "_BR"), Array As String("android.content.Context", "java.lang.String"))
	Return Obj2.RunMethod4( "getAppWidgetIds", Array As Object(cn), Array As String("android.content.ComponentName"))
End Sub

'Set a new click event for a view which includes the WidgetId so we can decide from which widget the event was fired.
'After setting an event with this sub the standard B4A event for this view will not work anymore.
'
' ServiceName - The name of the WidgetService
' WidgetId - Id of the widget
' rv - RemoteViews object
' ViewName - Name of the view for which the event is set
' EventName - The name of the event. A "_clicked" is appended to this event.
Public Sub SetWidgetClickEvent(ServiceName As String, WidgetId As Int, rv As RemoteViews, ViewName As String, EventName As String)
	Dim vIntent As Object
	Dim vPendingIntent As Object
	Dim ref As Reflector
	Dim ViewId As Int

	vIntent = ref.RunStaticMethod("anywheresoftware.b4a.keywords.Common", "getComponentIntent", Array As Object(ref.GetProcessBA(ServiceName), Null), Array As String("anywheresoftware.b4a.BA", "java.lang.Object"))
	ref.Target = vIntent
	EventName = EventName.ToLowerCase
	ref.RunMethod4("putExtra", Array As Object("b4a_internal_event", EventName & "_clicked"), Array As String("java.lang.String", "java.lang.String"))
	ref.RunMethod4("putExtra", Array As Object("appWidgetId", WidgetId), Array As String("java.lang.String", "java.lang.int"))

	ref.Target = rv
	ViewId = ref.RunMethod4("getIdForView", Array As Object(ref.GetProcessBA(ServiceName), ViewName), Array As String("anywheresoftware.b4a.BA", "java.lang.String"))

	vPendingIntent = ref.RunStaticMethod("android.app.PendingIntent", "getService", Array As Object(ref.GetContext, WidgetId, vIntent, 134217728), Array As String("android.content.Context", "java.lang.int", "android.content.Intent", "java.lang.int"))
	ref.RunMethod("checkNull")
	ref.Target = ref.GetField("current")
	ref.RunMethod4("setOnClickPendingIntent", Array As Object(ViewId, vPendingIntent), Array As String("java.lang.int", "android.app.PendingIntent"))
	
	UpdateSingleWidget(rv, WidgetId)
End Sub

'Load a new xml layout file for the widget. This will create completely new Remoteviews object for the widget.
'
'ServiceName - Name of the widget service
'pRv - RemoteViews object
'LayoutName - Name of the layout to load
Public Sub SetWidgetLayout(ServiceName As String, pRv As RemoteViews, LayoutName As String)
	Dim Obj1, Obj2 As Reflector
	Dim id As Int
	Dim newRV As Object
	Dim packagename As String
	
	packagename = GetPackageName
	id = Obj1.GetStaticField(packagename & ".R$layout", LayoutName)
	newRV = Obj1.CreateObject2("android.widget.RemoteViews", Array As Object(packagename, id), Array As String("java.lang.String", "java.lang.int"))

	Obj1.Target = pRv ' a RemoteViewsWrapper
	Obj2.Target = Obj1.GetField("original")
	Obj2.RunMethod4("setDataPosition", Array As Object(0), Array As String("java.lang.int"))
	Obj2.Target = newRV
	Obj2.RunMethod4("writeToParcel", Array As Object(Obj1.GetField("original"), 0), Array As String("android.os.Parcel", "java.lang.int"))
	Obj1.SetField2("current", Null)
End Sub

Sub DisallowScrolling(sv As ScrollView)
	Dim r As Reflector
	
	r.Target = sv
	r.RunMethod4("requestDisallowInterceptTouchEvent", Array As Object(True), Array As String("java.lang.boolean"))
End Sub


'Set the alpha value of a View on the widget.
'Works only with Android SDK version >=8
'
'ServiceName - Name of the widget service
'ViewName - Name of the view
'pRv - RemoteViews object
'pAlpha - Alpha value (0-255)
Sub SetRemoteViewAlpha(ServiceName As String, ViewName As String, pRv As RemoteViews, pAlpha As Int)
	Dim ref As Reflector
	Dim id As Int

	Dim p As Phone
	If p.SdkVersion >= 8 Then
		ref.Target = pRv
		id = ref.RunMethod4("getIdForView", Array As Object(ref.GetProcessBA(ServiceName), ViewName), Array As String("anywheresoftware.b4a.BA", "java.lang.String"))
		ref.RunMethod("checkNull")
		
		ref.Target = ref.GetField("current")
		ref.RunMethod4("setInt", Array As Object(id, "setAlpha", pAlpha), Array As String("java.lang.int", "java.lang.String", "java.lang.int"))
	Else
		Log("OS version too low, unable to set alpha")
	End If
End Sub

'Manually set the intent for the activity
Sub SetActivityIntent(pIntent As Intent)
	Dim ref As Reflector
	
	ref.Target = ref.GetActivity
	ref.RunMethod4("setIntent", Array As Object(pIntent), Array As String("android.content.Intent"))
End Sub

'Returns the package name of the app
Public Sub GetPackageName As String
    Dim r As Reflector
    Return r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
End Sub

Public Sub GetWallpaper As BitmapDrawable
	Dim r As Reflector
	
	r.Target = r.RunStaticMethod("android.app.WallpaperManager", "getInstance", Array As Object(r.GetContext), Array As String("android.content.Context"))
	Return r.RunMethod("getDrawable")
End Sub
