﻿Type=Activity
Version=2.52
@EndOfDesignText@
#Region  Activity Attributes 
	#FullScreen: False
	#IncludeTitle: True
#End Region

#Region Globals
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

	Dim ACTIVITY_RESULT_CANCELED As Int = 0
	Dim ACTIVITY_RESULT_OK As Int = -1
	
	Private gAppWidgetId As Int
	
	Type myLocation (Name As String, Latitude As Double, Longitude As Double, timezone As String)
	
	Private sax As SaxParser
	Private StartingIntent As Intent
	Private gSearchString As String
	
	Type PageInfo (PageCreated As Boolean)
End Sub

Sub Globals
	'These global variables will be redeclared each time the activity is created.
	'These variables can only be accessed from this module.

	Dim spTimeZone As Spinner
	Dim btSave As Button
	Dim eLocation As EditText
	Dim pTransparency As Panel
	Dim sTransparency As clsSeekBar
	Dim VP As AHViewPager
	Dim TB As AHViewPagerTabs
	Dim PC As AHPageContainer
	Dim pLayoutPreview As Panel
	Dim svMain As ScrollView
	Dim pBottom As Panel
	
	Private gTimeZone As String
	Private gLoc As myLocation
	
	Dim PageInfoMap As Map
	Dim btTextColor As Button
	Dim gTextColor As Int = Colors.White
End Sub
#End Region

#Region Activity
Sub Activity_Create(FirstTime As Boolean)
	If FirstTime Then StateManager.ResetState("WidgetConfig")	
	
	'Get the Intent we were started with and save it for later use.
	If Activity.GetStartingIntent.Action = "android.appwidget.action.APPWIDGET_CONFIGURE" Then
		StartingIntent = Activity.GetStartingIntent
	End If

	Activity.LoadLayout("laywidgetconfig")
	svMain.Panel.LoadLayout("conf")

	'If we are running on SDK-Version < 8 (Froyo) then it is not possible to set transparency
	Dim ph As Phone
	If ph.SdkVersion < 8 Then
		pTransparency.Visible = False
		pBottom.Top = spTimeZone.Top + spTimeZone.Height
	End If
	
	sTransparency.Initialize(Me, pTransparency, "STrans", 0, 100, "%")

	svMain.Panel.Height = pBottom.Top 'Set the scrollview panel height	

	'Fill the spinner with timezone entries.
	Dim tz As AHTimeZone	
	tz.Initialize
	spTimeZone.AddAll(tz.AvailableIds)
	
	If StartingIntent.HasExtra("appWidgetId") Then
		gAppWidgetId = StartingIntent.GetExtra("appWidgetId")
	Else
		gAppWidgetId = -1
	End If

	PageInfoMap.Initialize

	'Initialize the viewpager
	PC.Initialize
	For i = 0 To 3
		Dim p As Panel
		Dim title As String
		p.Initialize("")
		
		Select i
			Case 0
				title = "Clock right, Location, TimeZone"
			Case 1
				title = "Clock left, Location, TimeZone"
			Case 2
				title = "Clock, Date, Location"
			Case 3
				title = "Clock, Date"
		End Select
		
		PC.AddPage(p, title)
		Dim pi As PageInfo
		pi.Initialize
		pi.PageCreated = False
		PageInfoMap.Put(i, pi)
	Next

	VP.Initialize(PC, "VP")

	TB.Initialize(VP)
	TB.TabPaddingTop = 5dip
	TB.TabPaddingBottom = 5dip
	TB.TextColorCenter = Colors.RGB(51,181,229)
	TB.LineHeight = 0

	pLayoutPreview.AddView(TB, 0, 0, pLayoutPreview.Width, 40dip)
	pLayoutPreview.AddView(VP, 0, 40dip, pLayoutPreview.Width, pLayoutPreview.Height - 40dip)

	Dim bmd As BitmapDrawable
	Dim srcbm, bm As Bitmap
	Dim cv As Canvas
	Dim src, dest As Rect
	bm.InitializeMutable(VP.Width, VP.Height)
	bmd.Initialize(bm)
	cv.Initialize2(bm)
	srcbm = Util.GetWallpaper.Bitmap
	src.Initialize(Max(0,srcbm.Width - VP.Width) / 2, srcbm.Height / 2 - VP.Height / 2, Max(0,srcbm.Width - VP.Width) / 2 + VP.Width, srcbm.Height / 2 + VP.Height / 2)
	dest.Initialize(0, 0, VP.Width, VP.Height)
	cv.DrawBitmap(Util.GetWallpaper.Bitmap, src, dest)
	VP.Background = bmd

	Activity.AddMenuItem3("Search", "Menu", LoadBitmap(File.DirAssets, "2-action-search.png"), True)	
End Sub

Sub Activity_Resume
	'Initialize the result code with "CANCEL"
	Activity.SetActivityResult(ACTIVITY_RESULT_CANCELED, StartingIntent)

	'Check, if we were started from the search intent.
	Dim currentIntent As Intent
	currentIntent = Activity.GetStartingIntent
	If currentIntent.Action = "android.intent.action.SEARCH" Then
		'Intent is the searchintent, so do a location search
		If currentIntent.HasExtra("query") Then
			Dim keyboard As IME
			
			keyboard.Initialize("")
			keyboard.HideKeyboard
			SearchForLocation(currentIntent.GetExtra("query"))
		End If
	Else
		'Intent is the normal intent. So let the user configure the widget
		If StateManager.RestoreState(Activity, "WidgetConfig", 0) = False Then

			'Initialize the Views
			Dim tz As AHTimeZone
			Dim timezone As String
			Dim loc As AHLocale
			loc.Initialize
			tz.Initialize
			
			'Set default values
			eLocation.Text = StateManager.GetSetting2("cityname_" & gAppWidgetId, loc.DisplayCountry)
			timezone = StateManager.GetSetting2("timezone_" & gAppWidgetId, tz.ID)
			sTransparency.SetValue(ConvAlphaToTransp(StateManager.GetSetting2("transp_" & gAppWidgetId, 255)))
			gTextColor = StateManager.GetSetting2("textcolor_" & gAppWidgetId, Colors.White)
			btTextColor.Color = gTextColor
			VP.GotoPage(StateManager.GetSetting2("layout_" & gAppWidgetId, 0), False)
			
			For i = 0 To spTimeZone.Size -1
				If spTimeZone.GetItem(i) = timezone Then
					spTimeZone.SelectedIndex = i
					Exit
				End If
			Next
		Else
			VP.GotoPage(VP.CurrentPage, False)
		End If
		RefreshPreview(VP.CurrentPage)
	End If
End Sub

Sub Activity_Pause (UserClosed As Boolean)
	'Save the state of the Activity objects
	If UserClosed Then
		StateManager.ResetState("WidgetConfig")
	Else
		StateManager.SaveState(Activity, "WidgetConfig")
	End If
End Sub
#End Region

#Region ViewPager
'A page gets created. We create the layout of the preview page here. We use the ICOSImageView because it has the CENTER_INSIDE scale type.
'This means that the image will fit into the ImageView and aspect ratio is kept.
Sub VP_PageCreated (Position As Int, Page As Object)
	Dim pi As PageInfo
	pi = PageInfoMap.Get(Position)

	If Not(pi.PageCreated) Then
		Dim p, frame As Panel
		Dim vWidth, vHeight As Int
		Dim v As View
		p = Page
	
		frame.Initialize("")
		If Activity.Width > Activity.Height Then
			vWidth = 250dip
			vHeight = 45dip
		Else
			vWidth = 210dip
			vHeight = 75dip
		End If
		p.AddView(frame, (VP.Width - vWidth) / 2, (VP.Height - vHeight) / 2, vWidth, vHeight) 
		
		v = XmlViewEx.GetXmlLayout("advancedwidget_layout" & Position)
	
        frame.AddView(v, 0, 0, frame.Width, frame.Height)
		pi.PageCreated = True
		RefreshPreview(Position)
	End If
End Sub

Sub VP_PageChanged (Position As Int)
	RefreshPreview(Position)
End Sub

'On pagedestroy we remove all views from the page
'Sub VP_PageDestroyed (Position As Int, Page As Object)
'	Dim p As Panel
'	p = Page
'	
'	p.RemoveAllViews
'End Sub

Sub VP_PageScrolled (Position As Int, PositionOffset As Float, PositionOffsetPixels As Int)
	If PositionOffset > 0.5 Then PositionOffset = 1 - PositionOffset
	If PositionOffset > 0.02 Then Util.DisallowScrolling(svMain)
End Sub
#End Region

#Region Events
Sub btSave_Click
	'The user clicked "Save" so set the OK-Result
	Activity.SetActivityResult(ACTIVITY_RESULT_OK, StartingIntent)
	
	'Save the settings for the widgetId
	StateManager.SetSetting("cityname_" & gAppWidgetId, eLocation.Text)
	StateManager.SetSetting("timezone_" & gAppWidgetId, spTimeZone.GetItem(spTimeZone.SelectedIndex))
	StateManager.SetSetting("layout_" & gAppWidgetId, VP.CurrentPage)
	StateManager.SetSetting("transp_" & gAppWidgetId, ConvTranspToAlpha(sTransparency.GetValue))
	StateManager.SetSetting("textcolor_" & gAppWidgetId, gTextColor)
	StateManager.SaveSettings

	'We call the widget service manually here. Some launchers do this automatically, some don't.
	'I don't care. The worst thing that can happen is that the widget is updated twice after reconfiguration.
	Dim ServiceIntent As Intent
	ServiceIntent.Initialize("android.appwidget.action.APPWIDGET_UPDATE_OPTIONS", "")
	ServiceIntent.SetComponent(Util.GetPackageName & "/.advancedwidget")
	ServiceIntent.PutExtra("appWidgetId", gAppWidgetId)
	StartService(ServiceIntent)
	
	gAppWidgetId = -1
	Activity.Finish
End Sub

'When menu is selected manually start a search. The user can also press the search button.
Sub Menu_Click
	StartSearch
End Sub

Sub spTimeZone_ItemClick (Position As Int, Value As Object)
	RefreshPreview(VP.CurrentPage)
End Sub

Sub eLocation_TextChanged (Old As String, New As String)
	RefreshPreview(VP.CurrentPage)
End Sub

Sub btTextColor_Click
	Dim coldia As ColorDialog
	coldia.RGB = gTextColor
	If coldia.Show("Select Color", "Ok", "Cancel", "", Null) = DialogResponse.POSITIVE Then
		gTextColor = coldia.RGB
		btTextColor.Color = coldia.RGB
	End If
	RefreshPreview(VP.CurrentPage)
End Sub

Sub STrans_ValueChanged (Value As Int, UserChanged As Boolean)
	Dim pi As PageInfo
	If VP.IsInitialized Then
		pi = PageInfoMap.Get(VP.CurrentPage)
		If pi.PageCreated Then
			XmlViewEx.SetXmlViewAlpha(PC.GetPageObject(VP.CurrentPage), "advancedwidget_backimage", (100 - Value) / 100.0)
		End If
	End If
End Sub
#End Region

#Region Search
'This is a trick to manually show the standard Android search field.
Sub StartSearch
   Dim ref As Reflector
   ref.Target = ref.GetActivity
   ref.RunPublicmethod("onSearchRequested", Null, Null)
End Sub

'Do the real search for a location with geocoder
Sub SearchForLocation(SearchString As String)
	Dim geo As Geocoder
	gSearchString = SearchString
	geo.Initialize("GeoCoder")
	geo.GetFromLocationName(SearchString, 5, 1)
	
	ProgressDialogShow("Searching for " & SearchString)
End Sub

Sub GeoCoder_GeocodeDone(Address1() As Address, Tag As Object)
	Dim loca As myLocation

	'Sometimes the geocoder does not return a result (timeouts?). In this case we try to geocode the search string up to 5 times.
	If Address1.Length = 0 AND Tag < 5 Then
		Dim geo As Geocoder
		geo.Initialize("GeoCoder")
		geo.GetFromLocationName(gSearchString, 5, (Tag + 1))
		Return
	End If

	'Get some information from the resulting location.
	If Address1.Length > 0 Then
		For i = 0 To Address1.Length - 1
			If Address1(i).HasLatitude AND Address1(i).HasLongitude Then
				Dim loca As myLocation
				loca.Initialize
				
				loca.Latitude = Address1(i).Latitude
				loca.Longitude = Address1(i).Longitude
				If Address1(i).Locality <> Null Then
					loca.Name = Address1(i).Locality
				Else
					If Address1(i).FeatureName <> Null Then
						loca.Name = Address1(i).FeatureName
					Else
						If Address1(i).AddressLines <> Null AND Address1(i).AddressLines.Size > 0 Then
							loca.Name = Address1(i).AddressLines.Get(0)
						End If
					End If
				End If
							
				If Address1(i).CountryName <> Null Then
					loca.Name = loca.Name & " (" & Address1(i).CountryName & ")"
				End If
				Exit
			End If
		Next
	End If
	
	'If a location is found then Request timezone data from this location
	If loca.IsInitialized Then
		RequestTimeZone(loca)
	Else
		ProgressDialogHide
		ToastMessageShow("Unable to find location!", True)
	End If
End Sub

Sub RequestTimeZone(pLocation As myLocation)
	Dim job As HttpJob
	gLoc = pLocation

	'Requst timezone from geonames.org web service
	job.Initialize("TZRequest", Me)
	job.Download2("http://api.geonames.org/timezone", Array As String("lat", pLocation.Latitude, "lng", pLocation.Longitude, "username", "amberhome"))
	
	ProgressDialogShow("Getting TimeZone data for" & " " & pLocation.Name)
End Sub

Sub JobDone (Job As HttpJob)
	Log("JobName = " & Job.JobName & ", Success = " & Job.Success)

	If Job.JobName = "TZRequest" Then
		ProgressDialogHide
		If Job.Success = True Then	
			sax.Initialize
			sax.Parse(Job.GetInputStream, "SaxParser")
		Else
			gTimeZone = ""
		End If
		Job.Release
		
		Dim tz As AHTimeZone
		tz.Initialize2(gTimeZone)
		
		gLoc.timezone = gTimeZone
		
		eLocation.Text = gLoc.Name
		spTimeZone.SelectedIndex = spTimeZone.IndexOf(tz.ID)
		
		'Set the original Config Intent. This is done to prevent the Activity to search again after screen rotation
		Util.SetActivityIntent(StartingIntent)
		RefreshPreview(VP.CurrentPage)

'	Else If Job.JobName = "MapSearch" Then
'		ProgressDialogHide
'		If Job.Success = True Then
'			ParseJSONLocation(Job.GetString)
'		Else
'			'Msgbox(Util.GetText("Location not found!"), Util.GetText(C.TEXT_ERROR))	
'		End If
'		Job.Release
	End If
End Sub

Sub SaxParser_EndElement (Uri As String, Name As String, Text As StringBuilder)
	If sax.Parents.IndexOf("geonames") > -1 Then
		If sax.Parents.IndexOf("timezone") > -1 Then
			If Name = "timezoneId" Then
				gTimeZone = Text
			End If
		End If
	End If
End Sub
#End Region

#Region Helper
'Helper subs to convert percent value into a 0-255 range and vice versa
Sub ConvTranspToAlpha(Transp As Int) As Int
	Dim tmp As Int
	
	tmp = Transp / 100 * 255
	tmp = 255 - tmp
	Return tmp
End Sub

Sub ConvAlphaToTransp(Alpha As Int) As Int
	Dim tmp As Int
	
	tmp = Alpha / 255 * 100
	tmp = 100 - tmp
	Return tmp
End Sub
#End Region

#Region Refresh
Sub RefreshPreview(pPage As Int)
	Dim pi As PageInfo
	pi = PageInfoMap.Get(pPage)
	Log("Page: " & pPage)
	Log("Created: " & pi.PageCreated)
	If pi.PageCreated Then
		Log("Update Preview")
		Dim tz As AHTimeZone
		Dim du As AHDateUtils
		
		Dim vClock, vDate As String
		Log(spTimeZone.SelectedItem)
		tz.Initialize2(spTimeZone.SelectedItem)
		
		vClock = du.FormatDateTime(DateTime.Now+tz.GetOffset(DateTime.Now) , du.FORMAT_SHOW_TIME+du.FORMAT_UTC)
		vDate = du.FormatDateTime(DateTime.Now+tz.GetOffset(DateTime.Now), du.FORMAT_SHOW_WEEKDAY+du.FORMAT_SHOW_DATE+du.FORMAT_ABBREV_WEEKDAY+du.FORMAT_UTC)
		
		XmlViewEx.SetXmlViewText(PC.GetPageObject(pPage), "advancedwidget_llocation", eLocation.Text)
		XmlViewEx.SetXmlViewText(PC.GetPageObject(pPage), "advancedwidget_ltimezone", tz.ID)
		XmlViewEx.SetXmlViewText(PC.GetPageObject(pPage), "advancedwidget_lclock", vClock)
		XmlViewEx.SetXmlViewText(PC.GetPageObject(pPage), "advancedwidget_ldate", vDate)
		
		XmlViewEx.SetXmlViewTextColor(PC.GetPageObject(pPage), "advancedwidget_llocation", gTextColor)
		XmlViewEx.SetXmlViewTextColor(PC.GetPageObject(pPage), "advancedwidget_ltimezone", gTextColor)
		XmlViewEx.SetXmlViewTextColor(PC.GetPageObject(pPage), "advancedwidget_lclock", gTextColor)
		XmlViewEx.SetXmlViewTextColor(PC.GetPageObject(pPage), "advancedwidget_ldate", gTextColor)
		
		XmlViewEx.SetXmlViewAlpha(PC.GetPageObject(pPage), "advancedwidget_backimage", (100 - sTransparency.GetValue) / 100.0)
	End If
End Sub
#End Region

